@@ -55,13 +55,16 @@ class JobSpy : public QObject
 public:
     JobSpy(KIO::Job *job)
         : QObject(0),
-          m_spy(job, SIGNAL(result(KJob *))),
+          m_spy(job, SIGNAL(result(KJob*))),
           m_error(0)
     {
-        connect(job, &KJob::result, this, [this](KJob* job) { m_error = job->error(); });
+        connect(job, &KJob::result, this, [this](KJob * job) {
+            m_error = job->error();
+        });
     }
     // like job->exec(), but with a timeout (to avoid being stuck with a popup grabbing mouse and keyboard...)
-    bool waitForResult() {
+    bool waitForResult()
+    {
         // implementation taken from QTRY_COMPARE, to move the QVERIFY to the caller
         if (m_spy.isEmpty()) {
             QTest::qWait(0);
@@ -71,7 +74,10 @@ public:
         }
         return !m_spy.isEmpty();
     }
-    int error() const { return m_error; }
+    int error() const
+    {
+        return m_error;
+    }
 
 private:
     QSignalSpy m_spy;
@@ -96,7 +102,7 @@ private Q_SLOTS:
 
         QVERIFY(m_tempDir.isValid());
         QVERIFY(m_nonWritableTempDir.isValid());
-        QVERIFY(QFile(m_nonWritableTempDir.path()).setPermissions(QFile::ReadOwner|QFile::ReadUser|QFile::ExeOwner|QFile::ExeUser));
+        QVERIFY(QFile(m_nonWritableTempDir.path()).setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::ExeOwner | QFile::ExeUser));
         m_srcDir = m_tempDir.path();
 
         m_srcFile = m_srcDir + "/srcfile";
@@ -105,7 +111,7 @@ private Q_SLOTS:
 
     void cleanupTestCase()
     {
-        QVERIFY(QFile(m_nonWritableTempDir.path()).setPermissions(QFile::ReadOwner|QFile::ReadUser|QFile::WriteOwner|QFile::WriteUser|QFile::ExeOwner|QFile::ExeUser));
+        QVERIFY(QFile(m_nonWritableTempDir.path()).setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::WriteOwner | QFile::WriteUser | QFile::ExeOwner | QFile::ExeUser));
     }
 
     // Before every test method, ensure the test file m_srcFile exists
@@ -173,17 +179,17 @@ private Q_SLOTS:
         QTest::addColumn<bool>("shouldSourceStillExist");
 
         QTest::newRow("Ctrl") << Qt::KeyboardModifiers(Qt::ControlModifier) << Qt::CopyAction << m_srcFile << QString()
-            << 0 << true;
+                              << 0 << true;
         QTest::newRow("Shift") << Qt::KeyboardModifiers(Qt::ShiftModifier) << Qt::MoveAction << m_srcFile << QString()
-            << 0 << false;
+                               << 0 << false;
         QTest::newRow("Ctrl_Shift") << Qt::KeyboardModifiers(Qt::ControlModifier | Qt::ShiftModifier) << Qt::LinkAction << m_srcFile << QString()
-            << 0 << true;
+                                    << 0 << true;
         QTest::newRow("DropOnItself") << Qt::KeyboardModifiers() << Qt::CopyAction << m_srcDir << m_srcDir
-            << int(KIO::ERR_DROP_ON_ITSELF) << true;
+                                      << int(KIO::ERR_DROP_ON_ITSELF) << true;
         QTest::newRow("DropDirOnFile") << Qt::KeyboardModifiers(Qt::ControlModifier) << Qt::CopyAction << m_srcDir << m_srcFile
-            << int(KIO::ERR_ACCESS_DENIED) << true;
+                                       << int(KIO::ERR_ACCESS_DENIED) << true;
         QTest::newRow("NonWritableDest") << Qt::KeyboardModifiers() << Qt::CopyAction << m_srcFile << m_nonWritableTempDir.path()
-            << int(KIO::ERR_WRITE_ACCESS_DENIED) << true;
+                                         << int(KIO::ERR_WRITE_ACCESS_DENIED) << true;
     }
 
     void shouldDropToDirectory()
@@ -210,15 +216,17 @@ private Q_SLOTS:
         job->setUiDelegate(0);
         job->setUiDelegateExtension(0);
         JobSpy jobSpy(job);
-        QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
+        QSignalSpy itemCreatedSpy(job, SIGNAL(itemCreated(QUrl)));
 
         // Then the file is copied
         QVERIFY(jobSpy.waitForResult());
         QCOMPARE(jobSpy.error(), expectedError);
         if (expectedError == 0) {
+            QCOMPARE(copyJobSpy.count(), 1);
             const QString destFile = dest + "/srcfile";
-            QCOMPARE(spy.count(), 1);
-            QCOMPARE(spy.at(0).at(0).value<QUrl>(), QUrl::fromLocalFile(destFile));
+            QCOMPARE(itemCreatedSpy.count(), 1);
+            QCOMPARE(itemCreatedSpy.at(0).at(0).value<QUrl>(), QUrl::fromLocalFile(destFile));
             QVERIFY(QFile::exists(destFile));
             QCOMPARE(QFile::exists(m_srcFile), shouldSourceStillExist);
             if (dropAction == Qt::LinkAction) {
@@ -258,7 +266,8 @@ private Q_SLOTS:
         QDropEvent dropEvent(QPoint(10, 10), dropAction, &m_mimeData, Qt::LeftButton, modifiers);
         KIO::DropJob *job = KIO::drop(&dropEvent, QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
         job->setUiDelegate(0);
-        QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
+        QSignalSpy itemCreatedSpy(job, SIGNAL(itemCreated(QUrl)));
 
         // Then a confirmation dialog should appear
         PredefinedAnswerJobUiDelegate extension;
@@ -268,8 +277,9 @@ private Q_SLOTS:
         // and the file should be moved to the trash, no matter what the modifiers are
         QVERIFY2(job->exec(), qPrintable(job->errorString()));
         QCOMPARE(extension.m_askDeleteCalled, 1);
-        QCOMPARE(spy.count(), 1);
-        const QUrl trashUrl = spy.at(0).at(0).value<QUrl>();
+        QCOMPARE(copyJobSpy.count(), 1);
+        QCOMPARE(itemCreatedSpy.count(), 1);
+        const QUrl trashUrl = itemCreatedSpy.at(0).at(0).value<QUrl>();
         QCOMPARE(trashUrl.scheme(), QString("trash"));
         KIO::StatJob *statJob = KIO::stat(trashUrl, KIO::HideProgressInfo);
         QVERIFY(statJob->exec());
@@ -288,7 +298,7 @@ private Q_SLOTS:
         const QFile::Permissions origPerms = QFileInfo(m_srcFile).permissions();
         QVERIFY(QFileInfo(m_srcFile).isWritable());
         KIO::CopyJob *copyJob = KIO::move(QUrl::fromLocalFile(m_srcFile), QUrl(QStringLiteral("trash:/")));
-        QSignalSpy copyingDoneSpy(copyJob, SIGNAL(copyingDone(KIO::Job*, QUrl, QUrl, QDateTime, bool, bool)));
+        QSignalSpy copyingDoneSpy(copyJob, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
         QVERIFY(copyJob->exec());
         const QUrl trashUrl = copyingDoneSpy.at(0).at(2).value<QUrl>();
         QVERIFY(trashUrl.isValid());
@@ -299,10 +309,12 @@ private Q_SLOTS:
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, QUrl::fromLocalFile(m_srcDir), KIO::HideProgressInfo);
         job->setUiDelegate(0);
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QSignalSpy spy(job, SIGNAL(itemCreated(QUrl)));
 
         // Then the file should be moved, without a popup. No point in copying out of the trash, or linking to it.
         QVERIFY2(job->exec(), qPrintable(job->errorString()));
+        QCOMPARE(copyJobSpy.count(), 1);
         QCOMPARE(spy.count(), 1);
         QCOMPARE(spy.at(0).at(0).value<QUrl>(), QUrl::fromLocalFile(m_srcFile));
         QVERIFY(QFile::exists(m_srcFile));
@@ -329,9 +341,11 @@ private Q_SLOTS:
         QDropEvent dropEvent(QPoint(10, 10), Qt::CopyAction, &m_mimeData, Qt::LeftButton, Qt::NoModifier);
         KIO::DropJob *job = KIO::drop(&dropEvent, destUrl, KIO::HideProgressInfo);
         job->setUiDelegate(0);
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QVERIFY2(job->exec(), qPrintable(job->errorString()));
 
         // Then a full move shouldn't happen, just a link
+        QCOMPARE(copyJobSpy.count(), 1);
         const QStringList items = QDir(tempDestDir.path()).entryList();
         QVERIFY2(!items.contains("srcfile"), qPrintable(items.join(',')));
         QVERIFY2(items.contains("trash:" + QChar(0x2044) + ".desktop"), qPrintable(items.join(',')));
@@ -381,6 +395,7 @@ private Q_SLOTS:
         JobSpy jobSpy(job);
         qRegisterMetaType<KFileItemListProperties>();
         QSignalSpy spyShow(job, SIGNAL(popupMenuAboutToShow(KFileItemListProperties)));
+        QSignalSpy copyJobSpy(job, SIGNAL(copyJobStarted(KIO::CopyJob*)));
         QVERIFY(spyShow.isValid());
 
         // Then a popup should appear, with the expected available actions
@@ -400,6 +415,7 @@ private Q_SLOTS:
         QVERIFY(jobSpy.waitForResult());
         QCOMPARE(jobSpy.error(), expectedError);
         if (expectedError == 0) {
+            QCOMPARE(copyJobSpy.count(), 1);
             const QString destFile = dest + "/srcfile";
             QVERIFY(QFile::exists(destFile));
             QCOMPARE(QFile::exists(m_srcFile), shouldSourceStillExist);
@@ -448,15 +464,17 @@ private Q_SLOTS:
     }
 
 private:
-    static QMenu *findPopup() {
+    static QMenu *findPopup()
+    {
         Q_FOREACH (QWidget *widget, qApp->topLevelWidgets()) {
-            if (QMenu* menu = qobject_cast<QMenu*>(widget)) {
+            if (QMenu *menu = qobject_cast<QMenu *>(widget)) {
                 return menu;
             }
         }
         return Q_NULLPTR;
     }
-    static Qt::DropActions popupDropActions(QMenu *menu) {
+    static Qt::DropActions popupDropActions(QMenu *menu)
+    {
         Qt::DropActions actions;
         Q_FOREACH (QAction *action, menu->actions()) {
             const QVariant userData = action->data();
