@@ -44,18 +44,19 @@ QTEST_MAIN(KDirModelTest)
 #define exitLoop quit
 #endif
 
-#define connect(a,b,c,d) QVERIFY(QObject::connect(a,b,c,d))
+#define checkedConnect(a,b,c,d) QVERIFY(QObject::connect(a,b,c,d))
 
 #ifndef Q_OS_WIN
-#define SPECIALCHARS "specialchars%:.pdf"
+#define SPECIALCHARS " special chars%:.pdf"
 #else
-#define SPECIALCHARS "specialchars%.pdf"
+#define SPECIALCHARS " special chars%.pdf"
 #endif
 
 Q_DECLARE_METATYPE(KFileItemList)
 
 void KDirModelTest::initTestCase()
 {
+    qputenv("LC_ALL", "en_US.UTF-8");
     // To avoid a runtime dependency on klauncher
     qputenv("KDE_FORK_SLAVES", "yes");
 
@@ -65,9 +66,9 @@ void KDirModelTest::initTestCase()
     m_dirModel = 0;
     s_referenceTimeStamp = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
     m_tempDir = 0;
-    m_topLevelFileNames << "toplevelfile_1"
-                        << "toplevelfile_2"
-                        << "toplevelfile_3"
+    m_topLevelFileNames << QStringLiteral("toplevelfile_1")
+                        << QStringLiteral("toplevelfile_2")
+                        << QStringLiteral("toplevelfile_3")
                         << SPECIALCHARS
                         ;
     recreateTestData();
@@ -90,9 +91,9 @@ void KDirModelTest::recreateTestData()
      * PATH/toplevelfile_1
      * PATH/toplevelfile_2
      * PATH/toplevelfile_3
-     * PATH/specialchars%:.pdf
-     * PATH/.hidden
-     * PATH/.hidden2
+     * PATH/special chars%:.pdf
+     * PATH/.hiddenfile
+     * PATH/.hiddenfile2
      * PATH/subdir
      * PATH/subdir/testfile
      * PATH/subdir/testsymlink
@@ -103,8 +104,8 @@ void KDirModelTest::recreateTestData()
     foreach (const QString &f, m_topLevelFileNames) {
         createTestFile(path + f);
     }
-    createTestFile(path + ".hidden");
-    createTestFile(path + ".hidden2");
+    createTestFile(path + ".hiddenfile");
+    createTestFile(path + ".hiddenfile2");
     createTestDirectory(path + "subdir");
     createTestDirectory(path + "subdir/subsubdir", NoSymlink);
 
@@ -131,7 +132,7 @@ void KDirModelTest::fillModel(bool reload, bool expectAllIndexes)
     KDirLister *dirLister = m_dirModel->dirLister();
     qDebug() << "Calling openUrl";
     dirLister->openUrl(QUrl::fromLocalFile(path), reload ? KDirLister::Reload : KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     qDebug() << "enterLoop, waiting for completed()";
     enterLoop();
 
@@ -168,11 +169,11 @@ void KDirModelTest::collectKnownIndexes()
         QString fileName = item.url().fileName();
         if (item.isDir()) {
             m_dirIndex = idx;
-        } else if (fileName == "toplevelfile_1") {
+        } else if (fileName == QLatin1String("toplevelfile_1")) {
             m_fileIndex = idx;
-        } else if (fileName == "toplevelfile_2") {
+        } else if (fileName == QLatin1String("toplevelfile_2")) {
             m_secondFileIndex = idx;
-        } else if (fileName.startsWith("special")) {
+        } else if (fileName.startsWith(QLatin1String(" special"))) {
             m_specialFileIndex = idx;
         }
     }
@@ -194,7 +195,7 @@ void KDirModelTest::collectKnownIndexes()
         QModelIndex idx = m_dirModel->index(row, 0, m_dirIndex);
         if (m_dirModel->itemForIndex(idx).isDir()) {
             subdirIndex = idx;
-        } else if (m_dirModel->itemForIndex(idx).name() == "testfile") {
+        } else if (m_dirModel->itemForIndex(idx).name() == QLatin1String("testfile")) {
             m_fileInDirIndex = idx;
         }
     }
@@ -286,6 +287,15 @@ void KDirModelTest::testIndex()
 
     // Invalid sibling call
     QVERIFY(!m_dirModel->sibling(1, 0, m_fileInSubdirIndex).isValid());
+
+    // Test index() with a valid parent (dir).
+    QModelIndex index2 = m_dirModel->index(m_fileInSubdirIndex.row(), m_fileInSubdirIndex.column(), subsubdirIndex);
+    QVERIFY(index2.isValid());
+    QVERIFY(index2 == m_fileInSubdirIndex);
+
+    // Test index() with a non-parent (file).
+    QModelIndex index3 = m_dirModel->index(m_fileInSubdirIndex.row(), m_fileInSubdirIndex.column(), m_fileIndex);
+    QVERIFY(!index3.isValid());
 }
 
 void KDirModelTest::testNames()
@@ -364,9 +374,11 @@ void KDirModelTest::testIndexForItem()
 void KDirModelTest::testData()
 {
     // First file
+    QModelIndex idx1col0 = m_dirModel->index(m_fileIndex.row(), 0, QModelIndex());
+    QCOMPARE(idx1col0.data().toString(), QString("toplevelfile_1"));
     QModelIndex idx1col1 = m_dirModel->index(m_fileIndex.row(), 1, QModelIndex());
-    int size1 = m_dirModel->data(idx1col1, Qt::DisplayRole).toInt();
-    QCOMPARE(size1, 11);
+    QString size1 = m_dirModel->data(idx1col1, Qt::DisplayRole).toString();
+    QCOMPARE(size1, QString("11 B"));
 
     KFileItem item = m_dirModel->data(m_fileIndex, KDirModel::FileItemRole).value<KFileItem>();
     KFileItem fileItem = m_dirModel->itemForIndex(m_fileIndex);
@@ -410,8 +422,8 @@ void KDirModelTest::testModifyFile()
 #else
     ModelSpy modelSpy(m_dirModel);
 #endif
-    connect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     // "Touch" the file
     setTimeStamp(file, s_referenceTimeStamp.addSecs(20));
@@ -446,8 +458,8 @@ void KDirModelTest::testRenameFile()
     const QUrl newUrl = QUrl::fromLocalFile(m_tempDir->path() + "/toplevelfile_2_renamed");
 
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
-    connect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     KIO::SimpleJob *job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
     QVERIFY(job->exec());
@@ -483,7 +495,7 @@ void KDirModelTest::testRenameFile()
 
 void KDirModelTest::testMoveDirectory()
 {
-    testMoveDirectory("subdir");
+    testMoveDirectory(QStringLiteral("subdir"));
 }
 
 void KDirModelTest::testMoveDirectory(const QString &dir /*just a dir name, no slash*/)
@@ -495,8 +507,8 @@ void KDirModelTest::testMoveDirectory(const QString &dir /*just a dir name, no s
     const QString dest = destDir.path() + '/';
     QVERIFY(QDir(dest).exists());
 
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     // Move
     qDebug() << "Moving" << srcdir << "to" << dest;
@@ -514,8 +526,8 @@ void KDirModelTest::testMoveDirectory(const QString &dir /*just a dir name, no s
     QVERIFY(!m_dirModel->indexForUrl(QUrl::fromLocalFile(path + "subdir")).isValid());
     QVERIFY(!m_dirModel->indexForUrl(QUrl::fromLocalFile(path + "subdir_renamed")).isValid());
 
-    connect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     // Move back
     qDebug() << "Moving" << dest + dir << "back to" << srcdir;
@@ -545,14 +557,14 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
     delete m_dirModelForExpand;
     m_dirModelForExpand = new KDirModel;
     KDirLister *dirListerForExpand = m_dirModelForExpand->dirLister();
-    connect(dirListerForExpand, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirListerForExpand, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     dirListerForExpand->openUrl(url); // async
     enterLoop();
 
     // Now do the renaming
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
-    connect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+                   &m_eventLoop, SLOT(exitLoop()));
     KIO::SimpleJob *job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
     QVERIFY(job->exec());
 
@@ -581,7 +593,7 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
 
     // Test moving the renamed directory; if something inside KDirModel
     // wasn't properly updated by the renaming, this would detect it and crash (#180673)
-    testMoveDirectory("subdir_renamed");
+    testMoveDirectory(QStringLiteral("subdir_renamed"));
 
     // Put things back to normal
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
@@ -649,8 +661,8 @@ void KDirModelTest::testRenameDirectoryInCache() // #188807
 void KDirModelTest::testChmodDirectory() // #53397
 {
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
-    connect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+                   &m_eventLoop, SLOT(exitLoop()));
     const QString path = m_tempDir->path() + '/';
     KFileItem rootItem = m_dirModel->itemForIndex(QModelIndex());
     const mode_t origPerm = rootItem.permissions();
@@ -703,40 +715,40 @@ void KDirModelTest::testExpandToUrl_data()
     QTest::newRow(".")
             << int(NoFlag) << "." << (QStringList());
     QTest::newRow("subdir")
-            << int(NoFlag) << "subdir" << (QStringList() << "subdir");
+            << int(NoFlag) << "subdir" << (QStringList() << QStringLiteral("subdir"));
     QTest::newRow("subdir/.")
-            << int(NoFlag) << "subdir/." << (QStringList() << "subdir");
+            << int(NoFlag) << "subdir/." << (QStringList() << QStringLiteral("subdir"));
 
-    const QString subsubdir = "subdir/subsubdir";
+    const QString subsubdir = QStringLiteral("subdir/subsubdir");
     // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir.
     QTest::newRow("subdir/subsubdir")
-            << int(NoFlag) << subsubdir << (QStringList() << "subdir" << subsubdir);
+            << int(NoFlag) << subsubdir << (QStringList() << QStringLiteral("subdir") << subsubdir);
 
     // Must list root, emit expand for subdir, list subdir, emit expand for subsubdir, list subsubdir.
     const QString subsubdirfile = subsubdir + "/testfile";
     QTest::newRow("subdir/subsubdir/testfile sync")
-            << int(NoFlag) << subsubdirfile << (QStringList() << "subdir" << subsubdir << subsubdirfile);
+            << int(NoFlag) << subsubdirfile << (QStringList() << QStringLiteral("subdir") << subsubdir << subsubdirfile);
 
 #ifndef Q_OS_WIN
     // Expand a symlink to a directory (#219547)
     const QString dirlink = m_tempDir->path() + "/dirlink";
     createTestSymlink(dirlink, "/");
     QTest::newRow("dirlink")
-            << int(NoFlag) << "dirlink/tmp" << (QStringList() << "dirlink" << "dirlink/tmp");
+            << int(NoFlag) << "dirlink/tmp" << (QStringList() << QStringLiteral("dirlink") << QStringLiteral("dirlink/tmp"));
 #endif
 
     // Do a cold-cache test too, but nowadays it doesn't change anything anymore,
     // apart from testing different code paths inside KDirLister.
     QTest::newRow("subdir/subsubdir/testfile with reload")
-            << int(NewDir) << subsubdirfile << (QStringList() << "subdir" << subsubdir << subsubdirfile);
+            << int(NewDir) << subsubdirfile << (QStringList() << QStringLiteral("subdir") << subsubdir << subsubdirfile);
 
     QTest::newRow("hold dest dir") // #193364
-            << int(NewDir | ListFinalDir) << subsubdirfile << (QStringList() << "subdir" << subsubdir << subsubdirfile);
+            << int(NewDir | ListFinalDir) << subsubdirfile << (QStringList() << QStringLiteral("subdir") << subsubdir << subsubdirfile);
 
     // Put subdir in cache too (#175035)
     QTest::newRow("hold subdir and dest dir")
             << int(NewDir | CacheSubdir | ListFinalDir | Recreate) << subsubdirfile
-            << (QStringList() << "subdir" << subsubdir << subsubdirfile);
+            << (QStringList() << QStringLiteral("subdir") << subsubdir << subsubdirfile);
 
     // Make sure the last test has the Recreate option set, for the subsequent test methods.
 }
@@ -770,10 +782,10 @@ void KDirModelTest::testExpandToUrl()
     if (!m_dirModelForExpand || (flags & NewDir)) {
         delete m_dirModelForExpand;
         m_dirModelForExpand = new KDirModel;
-        connect(m_dirModelForExpand, SIGNAL(expand(QModelIndex)),
-                this, SLOT(slotExpand(QModelIndex)));
-        connect(m_dirModelForExpand, SIGNAL(rowsInserted(QModelIndex,int,int)),
-                this, SLOT(slotRowsInserted(QModelIndex,int,int)));
+        checkedConnect(m_dirModelForExpand, SIGNAL(expand(QModelIndex)),
+                       this, SLOT(slotExpand(QModelIndex)));
+        checkedConnect(m_dirModelForExpand, SIGNAL(rowsInserted(QModelIndex,int,int)),
+                       this, SLOT(slotRowsInserted(QModelIndex,int,int)));
         KDirLister *dirListerForExpand = m_dirModelForExpand->dirLister();
         dirListerForExpand->openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags); // async
     }
@@ -800,7 +812,7 @@ void KDirModelTest::testExpandToUrl()
     }
 
     // Now it should exist
-    if (!expandToPath.isEmpty() && expandToPath != ".") {
+    if (!expandToPath.isEmpty() && expandToPath != QLatin1String(".")) {
         qDebug() << "Do I know" << m_urlToExpandTo << "?";
         QVERIFY(m_dirModelForExpand->indexForUrl(m_urlToExpandTo).isValid());
     }
@@ -863,7 +875,7 @@ void KDirModelTest::testFilter()
     QSignalSpy spyItemsFilteredByMime(m_dirModel->dirLister(), SIGNAL(itemsFilteredByMime(KFileItemList)));
     QSignalSpy spyItemsDeleted(m_dirModel->dirLister(), SIGNAL(itemsDeleted(KFileItemList)));
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    m_dirModel->dirLister()->setNameFilter("toplevel*");
+    m_dirModel->dirLister()->setNameFilter(QStringLiteral("toplevel*"));
     QCOMPARE(m_dirModel->rowCount(), oldTopLevelRowCount); // no change yet
     QCOMPARE(m_dirModel->rowCount(m_dirIndex), oldSubdirRowCount); // no change yet
     m_dirModel->dirLister()->emitChanges();
@@ -884,17 +896,17 @@ void KDirModelTest::testFilter()
             const KFileItem item = m_dirModel->itemForIndex(parentIdx);
             dirName = item.name();
         } else {
-            dirName = "root";
+            dirName = QStringLiteral("root");
         }
         rowsRemovedPerDir[dirName] += args[2].toInt() - args[1].toInt() + 1;
         //qDebug() << parentIdx << args[1].toInt() << args[2].toInt();
     }
     QCOMPARE(rowsRemovedPerDir.count(), 3); // once for every dir
-    QCOMPARE(rowsRemovedPerDir.value("root"), 1);      // one from toplevel ('specialchars')
+    QCOMPARE(rowsRemovedPerDir.value("root"), 1);      // one from toplevel ('special chars')
     QCOMPARE(rowsRemovedPerDir.value("subdir"), 2);    // two from subdir
     QCOMPARE(rowsRemovedPerDir.value("subsubdir"), 1); // one from subsubdir
     QCOMPARE(spyItemsDeleted.count(), 3); // once for every dir
-    QCOMPARE(spyItemsDeleted[0][0].value<KFileItemList>().count(), 1); // one from toplevel ('specialchars')
+    QCOMPARE(spyItemsDeleted[0][0].value<KFileItemList>().count(), 1); // one from toplevel ('special chars')
     QCOMPARE(spyItemsDeleted[1][0].value<KFileItemList>().count(), 2); // two from subdir
     QCOMPARE(spyItemsDeleted[2][0].value<KFileItemList>().count(), 1); // one from subsubdir
     QCOMPARE(spyItemsFilteredByMime.count(), 0);
@@ -924,7 +936,7 @@ void KDirModelTest::testMimeFilter()
     QSignalSpy spyItemsFilteredByMime(m_dirModel->dirLister(), SIGNAL(itemsFilteredByMime(KFileItemList)));
     QSignalSpy spyItemsDeleted(m_dirModel->dirLister(), SIGNAL(itemsDeleted(KFileItemList)));
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    m_dirModel->dirLister()->setMimeFilter(QStringList() << "application/pdf");
+    m_dirModel->dirLister()->setMimeFilter(QStringList() << QStringLiteral("application/pdf"));
     QCOMPARE(m_dirModel->rowCount(), oldTopLevelRowCount); // no change yet
     QCOMPARE(m_dirModel->rowCount(m_dirIndex), oldSubdirRowCount); // no change yet
     m_dirModel->dirLister()->emitChanges();
@@ -999,10 +1011,10 @@ void KDirModelTest::testUrlWithRef() // #171117
     const QString path = m_tempDir->path() + '/';
     KDirLister *dirLister = m_dirModel->dirLister();
     QUrl url = QUrl::fromLocalFile(path);
-    url.setFragment("ref");
+    url.setFragment(QStringLiteral("ref"));
     QVERIFY(url.url().endsWith("#ref"));
     dirLister->openUrl(url, KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     enterLoop();
 
     QCOMPARE(dirLister->url().toString(), url.toString(QUrl::StripTrailingSlash));
@@ -1012,13 +1024,13 @@ void KDirModelTest::testUrlWithRef() // #171117
 
 void KDirModelTest::testFontUrlWithHost() // #160057
 {
-    if (!KProtocolInfo::isKnownProtocol("fonts")) {
+    if (!KProtocolInfo::isKnownProtocol(QStringLiteral("fonts"))) {
         QSKIP("kio_fonts not installed");
     }
-    QUrl url("fonts://foo/System");
+    QUrl url(QStringLiteral("fonts://foo/System"));
     KDirLister *dirLister = m_dirModel->dirLister();
     dirLister->openUrl(url, KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     enterLoop();
 
     QCOMPARE(dirLister->url().toString(), QString("fonts:/System"));
@@ -1026,13 +1038,13 @@ void KDirModelTest::testFontUrlWithHost() // #160057
 
 void KDirModelTest::testRemoteUrlWithHost() // #178416
 {
-    if (!KProtocolInfo::isKnownProtocol("remote")) {
+    if (!KProtocolInfo::isKnownProtocol(QStringLiteral("remote"))) {
         QSKIP("kio_remote not installed");
     }
-    QUrl url("remote://foo");
+    QUrl url(QStringLiteral("remote://foo"));
     KDirLister *dirLister = m_dirModel->dirLister();
     dirLister->openUrl(url, KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     enterLoop();
 
     QCOMPARE(dirLister->url().toString(), QString("remote:"));
@@ -1043,7 +1055,7 @@ void KDirModelTest::testZipFile() // # 171721
     const QString path = QFileInfo(QFINDTESTDATA("wronglocalsizes.zip")).absolutePath();
     KDirLister *dirLister = m_dirModel->dirLister();
     dirLister->openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     enterLoop();
     disconnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
 
@@ -1051,28 +1063,28 @@ void KDirModelTest::testZipFile() // # 171721
     zipUrl.setPath(zipUrl.path() + "/wronglocalsizes.zip"); // just a zip file lying here for other reasons
 
     QVERIFY(QFile::exists(zipUrl.toLocalFile()));
-    zipUrl.setScheme("zip");
+    zipUrl.setScheme(QStringLiteral("zip"));
     QModelIndex index = m_dirModel->indexForUrl(zipUrl);
     QVERIFY(!index.isValid()); // protocol mismatch, can't find it!
-    zipUrl.setScheme("file");
+    zipUrl.setScheme(QStringLiteral("file"));
     index = m_dirModel->indexForUrl(zipUrl);
     QVERIFY(index.isValid());
 }
 
 void KDirModelTest::testSmb()
 {
-    const QUrl smbUrl("smb:/");
+    const QUrl smbUrl(QStringLiteral("smb:/"));
     // TODO: feed a KDirModel without using a KDirLister.
     // Calling the slots directly.
     // This requires that KDirModel does not ask the KDirLister for its rootItem anymore,
     // but that KDirLister emits the root item whenever it changes.
-    if (!KProtocolInfo::isKnownProtocol("smb")) {
+    if (!KProtocolInfo::isKnownProtocol(QStringLiteral("smb"))) {
         QSKIP("kio_smb not installed");
     }
     KDirLister *dirLister = m_dirModel->dirLister();
     dirLister->openUrl(smbUrl, KDirLister::NoFlags);
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
-    connect(dirLister, SIGNAL(canceled()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(canceled()), this, SLOT(slotListingCompleted()));
     QSignalSpy spyCanceled(dirLister, SIGNAL(canceled()));
     enterLoop(); // wait for completed signal
 
@@ -1128,9 +1140,57 @@ void KDirModelTest::testMimeData()
     QCOMPARE(urls.count(), indexes.count());
 }
 
+void KDirModelTest::testDotHiddenFile_data()
+{
+    QTest::addColumn<QStringList>("fileContents");
+    QTest::addColumn<QStringList>("expectedListing");
+
+    QStringList allItems; allItems << QStringLiteral("toplevelfile_1") << QStringLiteral("toplevelfile_2") << QStringLiteral("toplevelfile_3") << SPECIALCHARS << QStringLiteral("subdir");
+    QTest::newRow("empty_file") << QStringList() << allItems;
+
+    QTest::newRow("simple_name") << (QStringList() << QStringLiteral("toplevelfile_1")) << QStringList(allItems.mid(1));
+
+    QStringList allButSpecialChars = allItems; allButSpecialChars.removeAt(3);
+    QTest::newRow("special_chars") << (QStringList() << SPECIALCHARS) << allButSpecialChars;
+
+    QStringList allButSubdir = allItems; allButSubdir.removeAt(4);
+    QTest::newRow("subdir") << (QStringList() << QStringLiteral("subdir")) << allButSubdir;
+
+    QTest::newRow("many_lines") << (QStringList() << QStringLiteral("subdir") << QStringLiteral("toplevelfile_1") << QStringLiteral("toplevelfile_3") << QStringLiteral("toplevelfile_2"))
+                                << (QStringList() << SPECIALCHARS);
+}
+
+void KDirModelTest::testDotHiddenFile()
+{
+    QFETCH(QStringList, fileContents);
+    QFETCH(QStringList, expectedListing);
+
+    const QString path = m_tempDir->path() + '/';
+    const QString dotHiddenFile = path + ".hidden";
+    QTest::qWait(1000); // mtime-based cache, so we need to wait for 1 second
+    QFile dh(dotHiddenFile);
+    QVERIFY(dh.open(QIODevice::WriteOnly));
+    dh.write(fileContents.join('\n').toUtf8());
+    dh.close();
+
+    // Do it twice: once to read from the file and once to use the cache
+    for (int i = 0; i < 2; ++i) {
+        fillModel(true, false);
+        QStringList files;
+        for (int row = 0; row < m_dirModel->rowCount(); ++row) {
+            files.append(m_dirModel->index(row, KDirModel::Name).data().toString());
+        }
+        files.sort();
+        expectedListing.sort();
+        QCOMPARE(files, expectedListing);
+    }
+
+    dh.remove();
+}
+
 void KDirModelTest::testDeleteFile()
 {
-    fillModel(false);
+    fillModel(true);
 
     QVERIFY(m_fileIndex.isValid());
     const int oldTopLevelRowCount = m_dirModel->rowCount();
@@ -1139,8 +1199,8 @@ void KDirModelTest::testDeleteFile()
     const QUrl url = QUrl::fromLocalFile(file);
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     KIO::DeleteJob *job = KIO::del(url, KIO::HideProgressInfo);
     QVERIFY(job->exec());
@@ -1181,7 +1241,7 @@ void KDirModelTest::testDeleteFileWhileListing() // doesn't really test that yet
 
     KDirLister *dirLister = m_dirModel->dirLister();
     QSignalSpy spyCompleted(dirLister, SIGNAL(completed()));
-    connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    checkedConnect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
     dirLister->openUrl(QUrl::fromLocalFile(path), KDirLister::NoFlags);
     if (!spyCompleted.isEmpty()) {
         QSKIP("listing completed too early");
@@ -1222,8 +1282,9 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
     const int oldTopLevelRowCount = m_dirModel->rowCount();
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
+    QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
 
     KIO::Job *job = KIO::move(QUrl::fromLocalFile(dir), QUrl::fromLocalFile(file), KIO::HideProgressInfo);
     job->setUiDelegate(0);
@@ -1234,11 +1295,31 @@ void KDirModelTest::testOverwriteFileWithDir() // #151851 c4
 
     QCOMPARE(extension.m_askFileRenameCalled, 1);
 
-    if (spyRowsRemoved.isEmpty()) {
-        // Wait for the DBUS signal from KDirNotify, it's the one the triggers rowsRemoved
-        enterLoop();
-        QVERIFY(!spyRowsRemoved.isEmpty());
+    // Wait for a removal within the top level (that's for the old file going away), and also
+    // for a dataChanged which notifies us that a file has become a directory
+    bool removalWithinTopLevel = false;
+    bool dataChangedAtFirstLevel = false;
+    int retries = 0;
+    while ((!removalWithinTopLevel || !dataChangedAtFirstLevel) && retries < 100) {
+        for (int i = 0; i < spyRowsRemoved.size() && !removalWithinTopLevel; ++i) {
+            QModelIndex parent = spyRowsRemoved[i][0].value<QModelIndex>();
+            if (!parent.isValid()) {
+                // yes, that's what we have been waiting for
+                removalWithinTopLevel = true;
+            }
+        }
+        for (int i = 0; i < spyDataChanged.size() && !dataChangedAtFirstLevel; ++i) {
+            QModelIndex idx = spyDataChanged[i][0].value<QModelIndex>();
+            if (idx.isValid() && !idx.parent().isValid()) {
+                // a change of a node whose parent is root, yay, that's it
+                dataChangedAtFirstLevel = true;
+            }
+        }
+        QTest::qWait(10);
+        ++retries;
     }
+    QVERIFY(removalWithinTopLevel);
+    QVERIFY(dataChangedAtFirstLevel);
 
     // If we come here, then rowsRemoved() was emitted - all good.
     const int topLevelRowCount = m_dirModel->rowCount();
@@ -1298,8 +1379,8 @@ void KDirModelTest::testRenameFileToHidden() // #174721
     QSignalSpy spyDataChanged(m_dirModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)));
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
     QSignalSpy spyRowsInserted(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)));
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     KIO::SimpleJob *job = KIO::rename(url, newUrl, KIO::HideProgressInfo);
     QVERIFY(job->exec());
@@ -1320,8 +1401,8 @@ void KDirModelTest::testRenameFileToHidden() // #174721
     spyRowsRemoved.clear();
 
     // Put things back to normal, should make the file reappear
-    connect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
     QVERIFY(job->exec());
     // Wait for the DBUS signal from KDirNotify, it's the one the triggers KDirLister
@@ -1341,8 +1422,8 @@ void KDirModelTest::testDeleteDirectory()
     const QUrl url = QUrl::fromLocalFile(path + "subdir/subsubdir");
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     QSignalSpy spyDirWatchDeleted(KDirWatch::self(), SIGNAL(deleted(QString)));
 
@@ -1373,8 +1454,8 @@ void KDirModelTest::testDeleteCurrentDirectory()
     const QUrl url = QUrl::fromLocalFile(path);
 
     QSignalSpy spyRowsRemoved(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)));
-    connect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            &m_eventLoop, SLOT(exitLoop()));
+    checkedConnect(m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+                   &m_eventLoop, SLOT(exitLoop()));
 
     KDirWatch::self()->statistics();
 
