@@ -21,6 +21,7 @@
 #include "kfileitem.h"
 
 #include "kioglobal_p.h"
+#include "kiocoredebug.h"
 
 #include <QtCore/QDate>
 #include <QtCore/QDir>
@@ -77,10 +78,6 @@ public:
         init();
     }
 
-    ~KFileItemPrivate()
-    {
-    }
-
     /**
      * Computes the text and mode from the UDSEntry
      * Called by constructor, but can be called again later
@@ -96,7 +93,6 @@ public:
     void setTime(KFileItem::FileTimes which, const QDateTime &val) const;
     bool cmp(const KFileItemPrivate &item) const;
     bool isSlow() const;
-    void stat();
 
     /**
      * Extracts the data from the UDSEntry member and updates the KFileItem
@@ -177,9 +173,6 @@ public:
     // For special case like link to dirs over FTP
     QString m_guessedMimeType;
     mutable QString m_access;
-
-    enum { NumFlags = KFileItem::CreationTime + 1 };
-    mutable QDateTime m_time[3];
 };
 
 void KFileItemPrivate::init()
@@ -187,10 +180,9 @@ void KFileItemPrivate::init()
     m_access.clear();
     //  metaInfo = KFileMetaInfo();
 
-    // determine mode and/or permissions if unknown
+    // stat() local files if needed
     // TODO: delay this until requested
-    if (m_fileMode == KFileItem::Unknown || m_permissions == KFileItem::Unknown) {
-        mode_t mode = 0;
+    if (m_fileMode == KFileItem::Unknown || m_permissions == KFileItem::Unknown || m_entry.count() == 0) {
         if (m_url.isLocalFile()) {
             /* directories may not have a slash at the end if
              * we want to stat() them; it requires that we
@@ -203,18 +195,29 @@ void KFileItemPrivate::init()
             const QString path = m_url.adjusted(QUrl::StripTrailingSlash).toLocalFile();
             const QByteArray pathBA = QFile::encodeName(path);
             if (QT_LSTAT(pathBA.constData(), &buf) == 0) {
-                mode = buf.st_mode;
+                m_entry.insert(KIO::UDSEntry::UDS_DEVICE_ID,           buf.st_dev);
+                m_entry.insert(KIO::UDSEntry::UDS_INODE,               buf.st_ino);
+
+                mode_t mode = buf.st_mode;
                 if ((buf.st_mode & QT_STAT_MASK) == QT_STAT_LNK) {
                     m_bLink = true;
                     if (QT_STAT(pathBA, &buf) == 0) {
                         mode = buf.st_mode;
-                    } else {// link pointing to nowhere (see FileProtocol::createUDSEntry() in kioslave/file/file.cpp)
+                    } else {// link pointing to nowhere (see FileProtocol::createUDSEntry() in ioslaves/file/file.cpp)
                         mode = (QT_STAT_MASK - 1) | S_IRWXU | S_IRWXG | S_IRWXO;
                     }
                 }
-                // While we're at it, store the times
-                setTime(KFileItem::ModificationTime, buf.st_mtime);
-                setTime(KFileItem::AccessTime, buf.st_atime);
+                m_entry.insert(KIO::UDSEntry::UDS_SIZE,      buf.st_size);
+                m_entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, buf.st_mode & QT_STAT_MASK); // extract file type
+                m_entry.insert(KIO::UDSEntry::UDS_ACCESS,    buf.st_mode & 07777); // extract permissions
+                m_entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME,   buf.st_mtime); // TODO: we could use msecs too...
+                m_entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME,         buf.st_atime);
+#ifndef Q_OS_WIN
+                m_entry.insert(KIO::UDSEntry::UDS_USER,                KUser(buf.st_uid).loginName());
+                m_entry.insert(KIO::UDSEntry::UDS_GROUP,               KUserGroup(buf.st_gid).name());
+#endif
+
+                // TODO: these can be removed, we can use UDS_FILE_TYPE and UDS_ACCESS everywhere
                 if (m_fileMode == KFileItem::Unknown) {
                     m_fileMode = mode & QT_STAT_MASK; // extract file type
                 }
@@ -287,48 +290,41 @@ KIO::filesize_t KFileItemPrivate::size() const
     return 0;
 }
 
+static uint udsFieldForTime(KFileItem::FileTimes mappedWhich)
+{
+    switch (mappedWhich) {
+    case KFileItem::ModificationTime:
+        return KIO::UDSEntry::UDS_MODIFICATION_TIME;
+    case KFileItem::AccessTime:
+        return KIO::UDSEntry::UDS_ACCESS_TIME;
+    case KFileItem::CreationTime:
+        return KIO::UDSEntry::UDS_CREATION_TIME;
+    }
+    return 0;
+}
+
 void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, uint time_t_val) const
 {
-    setTime(mappedWhich, QDateTime::fromTime_t(time_t_val));
+    m_entry.insert(udsFieldForTime(mappedWhich), time_t_val);
 }
 
 void KFileItemPrivate::setTime(KFileItem::FileTimes mappedWhich, const QDateTime &val) const
 {
-    m_time[mappedWhich] = val.toLocalTime(); // #160979
+    const QDateTime dt = val.toLocalTime(); // #160979
+    setTime(mappedWhich, dt.toTime_t());
 }
 
 QDateTime KFileItemPrivate::time(KFileItem::FileTimes mappedWhich) const
 {
-    if (!m_time[mappedWhich].isNull()) {
-        return m_time[mappedWhich];
-    }
-
     // Extract it from the KIO::UDSEntry
-    long long fieldVal = -1;
-    switch (mappedWhich) {
-    case KFileItem::ModificationTime:
-        fieldVal = m_entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1);
-        break;
-    case KFileItem::AccessTime:
-        fieldVal = m_entry.numberValue(KIO::UDSEntry::UDS_ACCESS_TIME, -1);
-        break;
-    case KFileItem::CreationTime:
-        fieldVal = m_entry.numberValue(KIO::UDSEntry::UDS_CREATION_TIME, -1);
-        break;
-    }
-    if (fieldVal != -1) {
-        setTime(mappedWhich, QDateTime::fromMSecsSinceEpoch(1000 * fieldVal));
-        return m_time[mappedWhich];
+    const uint uds = udsFieldForTime(mappedWhich);
+    if (uds > 0) {
+        const long long fieldVal = m_entry.numberValue(uds, -1);
+        if (fieldVal != -1) {
+            return QDateTime::fromMSecsSinceEpoch(1000 * fieldVal);
+        }
     }
 
-    // If not in the KIO::UDSEntry, or if UDSEntry empty, use stat() [if local URL]
-    if (m_bIsLocalUrl) {
-        QFileInfo info(localPath());
-        setTime(KFileItem::ModificationTime, info.lastModified());
-        setTime(KFileItem::AccessTime, info.lastRead());
-        setTime(KFileItem::CreationTime, info.created());
-        return m_time[mappedWhich];
-    }
     return QDateTime();
 }
 
@@ -347,7 +343,7 @@ bool KFileItemPrivate::cmp(const KFileItemPrivate &item) const
     //qDebug() << " m_bLink" << (m_bLink == item.m_bLink);
     //qDebug() << " m_hidden" << (m_hidden == item.m_hidden);
     //qDebug() << " size" << (size() == item.size());
-    //qDebug() << " ModificationTime" << (time(KFileItem::ModificationTime) == item.time(KFileItem::ModificationTime));
+    //qDebug() << " ModificationTime" << m_entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME) << item.m_entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME);
     //qDebug() << " UDS_ICON_NAME" << (m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME ));
 #endif
     return (m_strName == item.m_strName
@@ -360,7 +356,7 @@ bool KFileItemPrivate::cmp(const KFileItemPrivate &item) const
             && m_bLink == item.m_bLink
             && m_hidden == item.m_hidden
             && size() == item.size()
-            && time(KFileItem::ModificationTime) == item.time(KFileItem::ModificationTime) // TODO only if already known!
+            && m_entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME) == item.m_entry.numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME)
             && m_entry.stringValue(KIO::UDSEntry::UDS_ICON_NAME) == item.m_entry.stringValue(KIO::UDSEntry::UDS_ICON_NAME)
            );
 
@@ -468,14 +464,12 @@ KFileItem::KFileItem(mode_t mode, mode_t permissions, const QUrl &url, bool dela
     : d(new KFileItemPrivate(KIO::UDSEntry(), mode, permissions,
                              url, false, delayedMimeTypes))
 {
-    d->stat();
 }
 
 KFileItem::KFileItem(const QUrl &url, const QString &mimeType, mode_t mode)
     : d(new KFileItemPrivate(KIO::UDSEntry(), mode, KFileItem::Unknown,
                              url, false, false))
 {
-    d->stat();
     d->m_bMimeTypeKnown = !mimeType.isEmpty();
     if (d->m_bMimeTypeKnown) {
         QMimeDatabase db;
@@ -495,7 +489,7 @@ KFileItem::~KFileItem()
 void KFileItem::refresh()
 {
     if (!d) {
-        qWarning() << "null item";
+        qCWarning(KIO_CORE) << "null item";
         return;
     }
 
@@ -509,8 +503,7 @@ void KFileItem::refresh()
     // Clearing m_entry makes it possible to detect changes in the size of the file,
     // the time information, etc.
     d->m_entry.clear();
-    d->stat(); // re-populates d->m_entry
-    d->init();
+    d->init(); // re-populates d->m_entry
 }
 
 void KFileItem::refreshMimeType()
@@ -535,7 +528,7 @@ void KFileItem::setDelayedMimeTypes(bool b)
 void KFileItem::setUrl(const QUrl &url)
 {
     if (!d) {
-        qWarning() << "null item";
+        qCWarning(KIO_CORE) << "null item";
         return;
     }
 
@@ -543,10 +536,20 @@ void KFileItem::setUrl(const QUrl &url)
     setName(url.fileName());
 }
 
+void KFileItem::setLocalPath(const QString &path)
+{
+    if (!d) {
+        qCWarning(KIO_CORE) << "null item";
+        return;
+    }
+
+    d->m_entry.insert(KIO::UDSEntry::UDS_LOCAL_PATH, path);
+}
+
 void KFileItem::setName(const QString &name)
 {
     if (!d) {
-        qWarning() << "null item";
+        qCWarning(KIO_CORE) << "null item";
         return;
     }
 
@@ -688,22 +691,6 @@ bool KFileItemPrivate::isSlow() const
     return m_slow == Slow;
 }
 
-// This function should only be called when:
-// - KFileItem is called with a QUrl object directly!
-// - refresh is called.
-// This is done to re-populate the UDSEntry object.
-void KFileItemPrivate::stat()
-{
-    if(m_url.isLocalFile()) {
-        QT_STATBUF buf;
-        const QString path = m_url.adjusted(QUrl::StripTrailingSlash).toLocalFile();
-        const QByteArray pathBA = QFile::encodeName(path);
-        if (QT_LSTAT(pathBA.constData(), &buf) == 0) {
-            m_entry = KIO::UDSEntry(buf, m_url.fileName());
-        }
-    }
-}
-
 bool KFileItem::isSlow() const
 {
     if (!d) {
@@ -897,42 +884,6 @@ static QString iconFromDesktopFile(const QString &path)
     return icon;
 }
 
-static QMap<QString, QString> standardLocationsMap()
-{
-    static const
-            struct { QStandardPaths::StandardLocation location;
-                     QString name; }
-                   mapping[] = {
-            { QStandardPaths::MusicLocation, QStringLiteral("folder-music") },
-            { QStandardPaths::MoviesLocation, QStringLiteral("folder-videos") },
-            { QStandardPaths::PicturesLocation, QStringLiteral("folder-pictures") },
-            { QStandardPaths::TempLocation, QStringLiteral("folder-temp") },
-            { QStandardPaths::DownloadLocation, QStringLiteral("folder-download") },
-            // Order matters here as paths can be reused for multiple purposes
-            // We essentially want more generic choices to trump more specific
-            // ones.
-            // home > desktop > documents > *.
-            { QStandardPaths::DocumentsLocation, QStringLiteral("folder-documents") },
-            { QStandardPaths::DesktopLocation, QStringLiteral("user-desktop") },
-            { QStandardPaths::HomeLocation, QStringLiteral("user-home") } };
-    static const int count = sizeof mapping / sizeof *mapping;
-
-    QMap<QString, QString> map;
-    for (int i = 0 ; i < count; ++i) {
-        auto locations = QStandardPaths::standardLocations(mapping[i].location);
-        Q_FOREACH(const QString &location, locations) {
-            map.insert(location, mapping[i].name);
-        }
-    }
-    return map;
-}
-
-static QString iconForStandardPath(const QString &localDirectory)
-{
-    static auto map = standardLocationsMap();
-    return map.value(localDirectory, QString());
-}
-
 QString KFileItem::iconName() const
 {
     if (!d) {
@@ -980,7 +931,7 @@ QString KFileItem::iconName() const
             }
         }
 
-        d->m_iconName = iconForStandardPath(url.toLocalFile());
+        d->m_iconName = KIOPrivate::iconForStandardPath(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
             return d->m_iconName;
@@ -1026,7 +977,8 @@ QStringList KFileItem::overlays() const
         return QStringList();
     }
 
-    QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(',');
+    QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(',',QString::SkipEmptyParts);
+
     if (d->m_bLink) {
         names.append(QStringLiteral("emblem-symbolic-link"));
     }
@@ -1059,7 +1011,6 @@ QStringList KFileItem::overlays() const
     if (isHidden()) {
         names.append(QStringLiteral("hidden"));
     }
-
 #ifndef Q_OS_WIN
     if (((d->m_fileMode & QT_STAT_MASK) == QT_STAT_DIR) && d->m_bIsLocalUrl) {
         if (KSambaShare::instance()->isDirectoryShared(d->m_url.toLocalFile()) ||
@@ -1379,7 +1330,7 @@ QDataStream &operator>> (QDataStream &s, KFileItem &a)
     s >> strText;
 
     if (!a.d) {
-        qWarning() << "null item";
+        qCWarning(KIO_CORE) << "null item";
         return s;
     }
 
@@ -1617,10 +1568,12 @@ bool KFileItem::isRegularFile() const
 
 QDebug operator<<(QDebug stream, const KFileItem &item)
 {
+    QDebugStateSaver saver(stream);
+    stream.nospace();
     if (item.isNull()) {
         stream << "[null KFileItem]";
     } else {
-        stream << "[KFileItem for" << item.url() << "]";
+        stream << "[KFileItem for " << item.url() << "]";
     }
     return stream;
 }
