@@ -82,6 +82,11 @@ SlaveKeeper::SlaveKeeper()
     connect(&m_grimTimer, SIGNAL(timeout()), SLOT(grimReaper()));
 }
 
+SlaveKeeper::~SlaveKeeper()
+{
+    grimReaper();
+}
+
 void SlaveKeeper::returnSlave(Slave *slave)
 {
     Q_ASSERT(slave);
@@ -532,7 +537,7 @@ Slave *ProtoQueue::createSlave(const QString &protocol, SimpleJob *job, const QU
         scheduler()->connect(slave, SIGNAL(slaveStatus(qint64,QByteArray,QString,bool)),
                              SLOT(slotSlaveStatus(qint64,QByteArray,QString,bool)));
     } else {
-        qWarning() << "couldn't create slave:" << errortext;
+        qCWarning(KIO_CORE) << "couldn't create slave:" << errortext;
         if (job) {
             job->slotError(error, errortext);
         }
@@ -659,8 +664,8 @@ public:
             Q_FOREACH (Slave *slave, p->allSlaves()) {
                 slave->kill();
             }
-            p->deleteLater();
         }
+        qDeleteAll(m_protocols);
     }
     Scheduler *q;
 
