@@ -92,6 +92,14 @@ public:
     /// Return the info for a given trashed file
     bool infoForFile(int trashId, const QString &fileId, TrashedFileInfo &info);
 
+    struct TrashSpaceInfo {
+        qulonglong totalSize; // total trash size in bytes
+        qulonglong availableSize; // available trash space in bytes
+    };
+    /// Get the space info for a given trash path
+    /// Space information is only valid if trashSpaceInfo returns true
+    bool trashSpaceInfo(const QString &path, TrashSpaceInfo &info);
+
     /// Return the physicalPath for a given trashed file - helper method which
     /// encapsulates the call to infoForFile. Don't use if you need more info from TrashedFileInfo.
     QString physicalPath(int trashId, const QString &fileId, const QString &relativePath);
@@ -144,7 +152,11 @@ private:
     QString infoPath(int trashId, const QString &fileId) const;
     QString filesPath(int trashId, const QString &fileId) const;
 
+#ifdef Q_OS_OSX
+    int idForMountPoint(const QString &mountPoint) const;
+#else
     int idForDevice(const Solid::Device &device) const;
+#endif
     void refreshDevices() const;
 
     /// Find the trash dir to use for a given file to delete, based on original path
@@ -169,6 +181,14 @@ private Q_SLOTS:
     void jobFinished(KJob *job);
 
 private:
+    // delete the files and info subdirectories from all known trash directories
+    // (supposed to be empty!) to make sure OS X sees the trash as empty too.
+    // Stub except on OS X.
+    void deleteEmptyTrashInfrastructure();
+    // create the trash infrastructure; also called
+    // to recreate it on OS X.
+    bool createTrashInfrastructure(int trashId, const QString &path = QString());
+
     /// Last error code stored in class to simplify API.
     /// Note that this means almost no method can be const.
     int m_lastErrorCode;
